package player

import (
	"context"
	jsoniter "github.com/json-iterator/go"
	mhayaTime "github.com/mhaya/extend/time"
	"github.com/mhaya/game/game_cluster/internal/code"
	"github.com/mhaya/game/game_cluster/internal/data"
	"github.com/mhaya/game/game_cluster/internal/mdb"
	"github.com/mhaya/game/game_cluster/internal/mdb/models"
	"github.com/mhaya/game/game_cluster/internal/param"
)

func (p *actorPlayer) rank(req *param.GetRankReq) (*param.GetRankResp, int32) {
	if !p.isOnline {
		return nil, code.LoginError
	}
	var resp param.GetRankResp
	rankList := models.GetRankList(req.Id, req.Tp)
	resp.Ranks = rankList
	for _, rank := range rankList {
		if req.Tp == 1 {
			rank.IsClaim = p.Player.RankReward[req.Id].IsClaim
		}
		if p.Player.UserName == rank.UserName {
			resp.Self = rank
		}
	}
	if resp.Self == nil {
		ctx := context.Background()
		playerBaseKey := models.GetPlayBaseKey(p.Player.UserName)
		info, err := mdb.RDB.Get(ctx, playerBaseKey).Result()
		if err != nil {
			return nil, code.Error
		}
		var playerBase param.PlayerBase
		jsoniter.Unmarshal([]byte(info), &playerBase)
		key := models.GetRankKey(req.Id, req.Tp)
		resp.Self = &param.PlayerBase{
			UserName:      playerBase.UserName,
			UserNameMaybe: playerBase.UserNameMaybe,
			Avatar:        playerBase.Avatar,
			OpenId:        playerBase.OpenId,
			NickName:      playerBase.NickName,
			Rank:          models.GetPlayerRank(key, p.Player.UserName),
			Score:         models.GetPlayerRankScore(key, p.Player.UserName),
			Reward:        nil,
		}
	}
	return &resp, code.OK
}

func (p *actorPlayer) rankReward(req *param.GetRankRewardReq) (map[int]*data.ItemReward, int32) {
	if !p.isOnline {
		return nil, code.LoginError
	}

	var addItem = models.NewAddItem()
	if r, ok := p.Player.RankReward[req.Id]; ok {
		if r.IsClaim == 0 && len(r.Reward) > 0 {
			p.Player.RankReward[req.Id].IsClaim = 1
			p.Player.RankReward[req.Id].ClaimTime = mhayaTime.Now().Unix()
			addItem.Add(r.Reward)
			p.Player.Item.AddItem(r.Reward)
		}
	}
	p.Player.LevelUp()

	return addItem, code.OK
}
